#!/usr/bin/php -q
<?php

/*
    SNMP traffic metter by johnrembo@netzond.com
*/

if ($_SERVER['argc'] < 5) {
    echo "usage: snmp.php IP(snmp host) SNMP-COMMUNITY(public) OID(.1.3.6.1.2.1.2.2.1.10.1) TYPE(1 - gauge, 2 - counter)\n";
    die;
}

DEFINE('SNMP_TIMEOUT',500000);//[micro-seconds] snmp timeout period for unsuccesful queries in php-snmp mode
DEFINE('REFRESH_INTERVAL',1);//[seconds] snmp_traffic=(snmp_counter_old+snmp_counter_new)/refresh_interval
DEFINE('MULTIPLIER',1);//convert Bytes into Bits (set to 1 to disable conversion)
DEFINE('RETRIES',1);// query retries count in case of failure

/* 
    snmp query realization
    preferred order: 
    1. php snmp v2 get
    2. php snmp v1 get
    3. net-snmp
*/
function get_snmp($ip,$com,$oid,$timeout,$retries) {
    $snmp_status = get_extension_funcs("snmp");
    if ( isset($snmp_status) && is_array($snmp_status) ) {
	$snmp_status = array_flip($snmp_status);
	if ( isset($snmp_status['snmp2_get']) ) {//prefer snmp v2 (php5)
	    $result = @snmp2_get($ip,$com,$oid,$timeout,$retries);
	} else {//snmp v1 (php4)
	    $result = @snmpget($ip,$com,$oid,$timeout,$retries);
	}
    } else {//php_snmp module not available, use NET-SNMP
	$timeout = ($t=$timeout/1000000) < 1 ? 1 : $t;//force 1 sec. minimum if less.
	echo "timeout=$timeout\n";
	$snmp_binary = @exec("which snmpget");
	$result = @exec("$snmp_binary -O n -t $timeout -v 2c -c $com $ip $oid");
    }
    return $result;
}

/*
  $com - snmp community, $oid - snmp object indentifier
  returns array ( 0=>counter_value, 1 => query_time )
*/
function snmp_data($ip,$com,$oid) {
    $timePrev=time() + microtime();
    
    $counter=get_snmp($ip,$com,$oid,SNMP_TIMEOUT,RETRIES);
    
    $counter=trim(substr($counter,strrpos($counter,":")+2));

    $timeNow=time() + microtime() - $timePrev;
    $result[]=$counter;
    $result[]=$timeNow;
    
    return $result;
}

$ip = $_SERVER['argv'][1];	//IP query is being sent to
$com = $_SERVER['argv'][2];	//SNMP COMMUNITY
$oid = $_SERVER['argv'][3];	//SNMP OID
$type = $_SERVER['argv'][4];	//DATA TYPE - counter or gauge

if ($type == 1) {

    $response =  snmp_data($ip,$com,$oid);
    echo $response[0];

} else {

    $counter['prev']=snmp_data($ip,$com,$oid);
    sleep(REFRESH_INTERVAL);
    $counter['curr']=snmp_data($ip,$com,$oid);
    $result=round(MULTIPLIER*($counter['curr'][0] - $counter['prev'][0]) / ($counter['prev'][1]/2+$counter['curr'][1]/2+REFRESH_INTERVAL),2)."\n";
    echo $result;

}

?>
